@@ -9,7 +9,7 @@ from einops import rearrange, repeat, reduce
 from einops.layers.torch import Rearrange
 
 import alphafold2_pytorch.constants as constants
-from alphafold2_pytorch.utils import get_bucketed_distance_matrix, center_distogram_torch, MDScaling, scn_backbone_mask, scn_cloud_mask
+from alphafold2_pytorch.utils import get_bucketed_distance_matrix, center_distogram_torch, MDScaling, scn_backbone_mask, scn_cloud_mask, sidechain_container
 from alphafold2_pytorch.reversible import ReversibleSequence
 
 # structure module
@@ -693,7 +693,13 @@ class Alphafold2(nn.Module):
 
         if self.num_backbone_atoms == 3:
             N_mask, CA_mask, C_mask = scn_backbone_mask(seq, boolean = True)
-            mask = repeat(mask, 'b n -> b (l n)', l = 3)
+
+            cloud_mask = scn_cloud_mask(seq, boolean = True)
+            flat_cloud_mask = rearrange(cloud_mask, 'b l c -> b (l c)')
+            chain_mask = (mask.unsqueeze(-1) * cloud_mask)
+            flat_chain_mask = rearrange(chain_mask, 'b l c -> b (l c)')
+
+            mask = rearrange(chain_mask[:, :, :self.num_backbone_atoms], 'b l c -> b (l c)')
 
         # structural refinement
 
@@ -701,13 +707,16 @@ class Alphafold2(nn.Module):
         coords_3d, _ = MDScaling(distances, 
             weights = weights,
             iters = self.mds_iters,
-            fix_mirror = 0,
+            fix_mirror = True,
             N_mask = N_mask,
             CA_mask = CA_mask,
             C_mask = C_mask
         )
-
         coords = rearrange(coords_3d, 'b c n -> b n c')
+        coords = sidechain_container(coords, n_aa = self.num_backbone_atoms, cloud_mask=cloud_mask)
+        coords = rearrange(coords[:, :, :self.num_backbone_atoms], 'b n l d -> b (n l) d')
+        # atom_tokens = scn_atom_embedd(seq)
+
         
         trunk_embeds = self.trunk_to_structure_dim(trunk_embeds)
         x = reduce(trunk_embeds, 'b i j d -> b i d', 'mean')
@@ -726,3 +735,5 @@ class Alphafold2(nn.Module):
 
         coords.type(original_dtype)
         return coords, mask
+
+
